from wikigeo.wikisource.wikitext import WikiText  
import time

class TestWikiscraper:

    def test_scrape(self):
        wiki = WikiText(100, 'de')
        start = time.time()
        result = wiki.scrape_page_text('Berlin')
        assert isinstance(result['text'], str) and len(result['text']) <= 100
    
    def translate_one(self):
        wiki = WikiText(100, 'de')
        testtext: """
        Berlin besitzt neben ausgedehnten Waldgebieten im Westen und Südosten des Stadtgebietes (Berliner Forsten) 
        viele große Parkanlagen. Da auch fast alle Straßen von Bäumen gesäumt sind, gilt Berlin als besonders grüne
         Stadt. In Berlin gibt es insgesamt rund 440.000 Straßenbäume, darunter 153.000 Linden, 82.000 Ahornbäume, 
         35.000 Eichen, 25.000 Platanen und 21.000 Kastanien.[25] Die über 2500 öffentlichen Grün-, Erholungs- und 
         Parkanlagen haben eine Gesamtfläche von über 5500 Hektar und bieten vielfältige Freizeit- und 
         Erholungsmöglichkeiten. Die größte heute als Park bezeichnete Anlage Berlins ist der Tempelhofer Park, der 
         auf dem ehemaligen Flughafen Tempelhof entstand. Im Zentrum der Stadt liegt der Große Tiergarten. Er ist die 
         älteste und mit 210 Hektar zweitgrößte und bedeutendste Parkanlage Berlins und wurde im Verlauf von mehr als 
         500 Jahren gestaltet. Ursprünglich ein ausgedehntes Waldareal vor den Toren der Stadt, genutzt von den 
         preußischen Adeligen als Jagd- und Ausrittgebiet, wurde dieses nach und nach von der Stadtentwicklung 
         umschlossen. Er erstreckt sich heute vom Bahnhof Zoo bis zum Brandenburger Tor und grenzt direkt an das 
         Regierungsviertel. Einige große Straßen durchschneiden den Tiergarten, darunter die Straße des 17. Juni als
          Ost-West-Achse. Sie kreuzen sich am Großen Stern, in dessen Mitte seit 1939 die Siegessäule steht. Der Große 
          Tiergarten hat die Gestalt einer naturnahen Parklandschaft: Charakteristisch sind die weiten, von kleinen 
          Wasserläufen durchzogenen und mit Baumgruppen bestandenen Rasenflächen sowie die Seen mit kleinen Inseln und 
          zahlreichen Brücken und Alleen. Anlagen wie der Englische Garten, die Luiseninsel und der Rosengarten setzen 
          an einigen Stellen schmuckgärtnerische Akzente.
        """
        results = wiki.translate_text([testtext], 'en')
        for result in results:
            assert isinstance(result, str) and len(result) <= 100
    
    def translate_many(self):
        wiki = WikiText(100, 'de')
        testtext: """
        Berlin besitzt neben ausgedehnten Waldgebieten im Westen und Südosten des Stadtgebietes (Berliner Forsten) 
        viele große Parkanlagen. Da auch fast alle Straßen von Bäumen gesäumt sind, gilt Berlin als besonders grüne
         Stadt. In Berlin gibt es insgesamt rund 440.000 Straßenbäume, darunter 153.000 Linden, 82.000 Ahornbäume, 
         35.000 Eichen, 25.000 Platanen und 21.000 Kastanien.[25] Die über 2500 öffentlichen Grün-, Erholungs- und 
         Parkanlagen haben eine Gesamtfläche von über 5500 Hektar und bieten vielfältige Freizeit- und 
         Erholungsmöglichkeiten. Die größte heute als Park bezeichnete Anlage Berlins ist der Tempelhofer Park, der 
         auf dem ehemaligen Flughafen Tempelhof entstand. Im Zentrum der Stadt liegt der Große Tiergarten. Er ist die 
         älteste und mit 210 Hektar zweitgrößte und bedeutendste Parkanlage Berlins und wurde im Verlauf von mehr als 
         500 Jahren gestaltet. Ursprünglich ein ausgedehntes Waldareal vor den Toren der Stadt, genutzt von den 
         preußischen Adeligen als Jagd- und Ausrittgebiet, wurde dieses nach und nach von der Stadtentwicklung 
         umschlossen. Er erstreckt sich heute vom Bahnhof Zoo bis zum Brandenburger Tor und grenzt direkt an das 
         Regierungsviertel. Einige große Straßen durchschneiden den Tiergarten, darunter die Straße des 17. Juni als
          Ost-West-Achse. Sie kreuzen sich am Großen Stern, in dessen Mitte seit 1939 die Siegessäule steht. Der Große 
          Tiergarten hat die Gestalt einer naturnahen Parklandschaft: Charakteristisch sind die weiten, von kleinen 
          Wasserläufen durchzogenen und mit Baumgruppen bestandenen Rasenflächen sowie die Seen mit kleinen Inseln und 
          zahlreichen Brücken und Alleen. Anlagen wie der Englische Garten, die Luiseninsel und der Rosengarten setzen 
          an einigen Stellen schmuckgärtnerische Akzente.
        """
        results = wiki.translate_text([testtext, testtext, testtext], 'en')
        for result in results:
            assert isinstance(result, str) and len(result) <= 100


from requests_html import HTML, HTMLSession
import time
from googletrans import Translator
import logging

class WikiText(object):

    """methods to scrape and parse text from wikipedia pages
    (quicker than using the parse api)"""

    def __init__(self, limit, language='en'):
        self.wikidomain = 'https://{}.wikipedia.org/wiki/'.format(language)
        self.data = {}
        self.language = language
        self.translator = Translator()
        self.limit = limit
        if(not isinstance(self.limit, int)):
            raise Exception('invalid limit argument; must be False or int')
    
    def scrape_page_text(self, pagetitle):
        """returns text on a given wikipedia page. 

        pagetitle: string of the extact page title"""

        session = HTMLSession()
        url = self.wikidomain + pagetitle.replace(' ', '_')
        response = session.get(url)
        if(not response.ok):
            raise Exception('error response:' + response)
        logging.debug(response)
        page = response.html
        pagetext = [elem.text for elem in page.find('p, h2, h3')]
        pagetext = ' '.join(pagetext).replace('[edit]', '')
        if(self.limit):
            result = {'title': pagetitle, 'text': pagetext[0:self.limit]}
        else:
            result = {'title': pagetitle, 'text': pagetext}
        return result

    def translate_text(self, textlist, translateto):
        """translates text into the specified language. 
        e.g. if using the french (fr) wikipedia, can translate results into english by setting
         translateto='en'
         textlist: a list of strings to be translated. each string must be under 10000 chars
         returns a list of strings of translated text"""
        
        for extract in textlist:
            if(len(extract) > 10000):
                raise Exception('Texts over 10000 chars are not reliably translated. Please use limit parameter.')
        translations = self.translator.translate(textlist, dest=translateto, src=self.language)
        if(self.limit):
            result = [translation.text[0:self.limit] for translation in translations]
        else:
            result = [translation.text for translation in translations]
        return result




import pytest
from wikigeo import ConcurrentSearcher

def test_nearby_pages():
    searcher = ConcurrentSearcher('en', "testing (marymcguire1718@gmail.com)")
    coords = [
        (51.19482552438797, -0.4718933366807213),
        (51.374798908124774, -0.7340422319483855),
        (51.4368078571564, -0.6332591317376522),
        (51.655000856037724, 0.0462537139415177),
        (51.18393717250045, -0.825727955629195),
        (51.39861787462966, -1.6807673580284554),
        (51.28843277725252, 0.2476129201376236),
        (51.060624488352296, -0.1604187830896187),
        (51.18583407040071, -0.4277369182879632),
        (51.17760083671381, -0.4678810897599431),
        (51.31329596922965, -0.8473433886048575),
        (50.86854995219705, -0.595084718328369)
    ]
    results = searcher.multi_nearby_pages(coords, limit=5, radiusmetres=8000)
    for result in results:
        print(result)
        assert isinstance(result['coords'], tuple) and isinstance(result['result'][0]['title'], str)

def test_page_text_no_translate():
    searcher = ConcurrentSearcher('en', "testing (marymcguire1718@gmail.com)")
    titles = ['Staines-upon-Thames', 'Edinburgh', 'Killeter', 'Northern Ireland', 'Staines Moor', 
    'Epping Forest', 'France', 'Greater_London', 'Paris', 'Scotland', 'England', 'Manchester', 'Hyde Park', 
    'Birmingham', 'Surrey', 'Kent', 'South_East_England', 'Berkshire', 'City of London', 'Central_London']
    result = searcher.multi_page_text(titles, textlen=50, translateto=False)
    for page in result:
        assert isinstance(page['text'][0], str) and len(page['text']) == 50

def test_page_text_translate():
    searcher = ConcurrentSearcher('en', "testing (marymcguire1718@gmail.com)")
    titles = ['Staines-upon-Thames', 'Edinburgh', 'Berlin', 'Northern Ireland', 'Staines Moor', 
    'Epping Forest', 'France', 'Greater_London', 'Paris', 'Scotland', 'England', 'Manchester', 'Hyde Park', 
    'Birmingham', 'Surrey', 'Kent', 'South_East_England', 'Berkshire', 'City of London', 'Central_London']

    result = searcher.multi_page_text(titles, textlen=50, translateto='fr')
    for page in result:
        assert isinstance(page['text'], str) and len(page['text']) <= 50

def test_nearby_images():
    searcher = ConcurrentSearcher('en', "testing (marymcguire1718@gmail.com)")
    coords = [(55.94631, -3.16552), (51.43399588542416, -0.5114918947219849)]
    names = ["Edinburgh", False]
    imagesdata = searcher.multi_nearby_images(coords, names, matchfilter=40)
    for images in imagesdata:
        print(images)
        for result in images['result']:
            assert isinstance(result, dict) and isinstance(result['image'], str)

def test_page_match():
    searcher = ConcurrentSearcher('en', "testing (marymcguire1718@gmail.com)")
    searches = [("Arthur's Seat", 55.94631, -3.16552), ('Staines', 51.43399588542416, -0.5114918947219849)]
    matches = searcher.multi_page_match(searches, bestmatch=True, minnamematch=0)
    for page in matches:
        print(page)
        assert isinstance(page['result'][0]['title'], str) and isinstance(page['result'][0]['lat'], float)



import requests_html as r
from requests_html import HTML
import json
import math
from fuzzywuzzy import fuzz
import logging
import time


class WikipediaAPI(object):

    """sends queries to the Wikipedia API
    language options and stats can be found here: https://en.wikipedia.org/wiki/List_of_Wikipedias"""

    def __init__(self, userinfo, language='en'):
        self.headers = {"User-agent": userinfo}
        self.session = r.HTMLSession()
        self.url = "https://{}.wikipedia.org/w/api.php".format(language)
        self.data = None
        self.query = None

    def _send_query(self):
        """sends a query and saves response"""

        response = self.session.get(self.url, params=self.query, headers=self.headers)
        logging.debug(response)
        if(not response.ok):
            raise Exception(response)
        self.data = response.json()
        try:
            logging.warning(self.data['error'])
            raise Exception('Check parameters; ' + str(self.data))
        except KeyError:
            pass
    
    def _next_search_results(self):
        """gets next set of results for query response"""

        current_results = self.data
        while('continue' in current_results):
            logging.debug('getting next page')
            _continue = current_results['continue']
            self.query['continue'] = _continue
            self._send_query
            time.sleep(1)
            if(self.data == current_results):
                logging.debug('reached end of results')
                break
            for page, result in self.data['query']['pages'].items():
                try:
                    current_results['query']['pages'][page].update(result)
                except:
                    logging.warning(str(page) + 'missing from new results')
            self.data = current_results

    def search_nearby(self, lat, lon, limit, radiusmetres):
        """query to get wiki pages near a coordinate
        options for geosearch are found here: 
        https://en.wikipedia.org/w/api.php?action=help&modules=query+geosearch"""

        if(not isinstance(lat, float)):
            raise Exception('Check parameters; latitude must be a float')
        if(not isinstance(lon, float)):
            raise Exception('Check parameters; longitude must be a float')
        if((not isinstance(radiusmetres, int)) or (not 10 <= radiusmetres <= 10000)):
            raise Exception('Check parameters; radiusmetres must be an int between 10 and 10000')
        if((not isinstance(limit, int)) or (not 0 < limit <= 500)):
            raise Exception('Check parameters; limit must be an int between 1 and 500')

        self.query = {
        "format": "json",
        "generator": "geosearch",
        "ggscoord": "{}|{}".format(lat, lon),
        "ggslimit": "{}".format(limit),
        "ggsradius": "{}".format(radiusmetres),
        "action": "query",
        "prop": "coordinates|pageterms|pageimages"
        }
        
        self._send_query()
        self._next_search_results()
    
    def search_string(self, search_string, limit=5):
        """query to search wikipedia for pages containing search string.
        options for search found here: https://www.mediawiki.org/wiki/API:Search
        and for query here: https://www.mediawiki.org/wiki/API:Query"""
        
        if(not isinstance(search_string, str)):
            raise Exception('Check parameters; keyword must be a string')
        if((not isinstance(limit, int)) or (not 0 < limit <= 500)):
            raise Exception('Check parameters; limit must be an int between 1 and 500')

        self.query = {
            "format": "json",
            'generator': 'search',
            "gsrsearch": search_string.replace(' ', '+'),
            'gsrlimit': '{}'.format(limit),
            "action": "query",
            'prop': 'coordinates|pageterms|pageimages',
            'piprop': 'original|name',
            'coprop': 'type',
            'coprimary': 'all'
        }
        self._send_query()
        self._next_search_results()

    """
    def parse_page(self, pagetitle, to_parse=['text']):
        #query to parse a wiki page by page title.
        #options for what to parse can be found here:
         #https://www.mediawiki.org/wiki/API:Parsing_wikitext

        if(not isinstance(pagetitle, str)):
            raise Exception('Check parameters; pagetitle must be a string')
        if(not isinstance(to_parse, list)):
            raise Exception('Check parameters; to_parse must be a list of strings to parse')

        self.query = {
            "format": "json",
            "action": "parse",
            "page": '{}'.format(pagetitle),
            "prop": "{}".format('|'.join(to_parse)),
        }
        self._send_query()
    """

    def return_data(self):
        """output response data"""
        return self.data
    
class WikiCommonsAPI(WikipediaAPI):

    """sends queries to the wiki commons api"""

    def __init__(self, userinfo):
        super().__init__(userinfo)
        self.url = "https://commons.wikimedia.org/w/api.php"
    
    def search_nearby(self, lat, lon, radiusmetres):
        """get images near given coordinates from wikimedia commons.
        options for image info are found here:
         https://www.mediawiki.org/w/api.php?action=help&modules=query%2Bimageinfo"""

        if(not isinstance(lat, float)):
            raise Exception('Check parameters; latitude must be a float')
        if(not isinstance(lon, float)):
            raise Exception('Check parameters; longitude must be a float')
        if((not isinstance(radiusmetres, int)) or (not 10 <= radiusmetres <= 10000)):
            raise Exception('Check parameters; radiusmetres must be an int between 10 and 10000')

        self.query = {
            'format': 'json',
            'action': 'query',
            'generator': 'geosearch',
            'ggscoord': "{}|{}".format(lat, lon),
            'ggslimit': 5,
            'prop': 'imageinfo|imagelabels',
            'iilimit': 1,
            'iiprop': 'url',
            'iiurlwidth': '250',
            'iiurlheight': '250',
            'ggsradius': "{}".format(radiusmetres),
            'ggsnamespace': '6',
            'ggsprimary': 'all'
        }
        self._send_query()
        self._next_search_results()




import pytest
from wikigeo import WikiExtractor


def test_get_nearby_images():
    wiki = WikiExtractor(language='en', userinfo="test")
    images = wiki.get_nearby_images(55.95527, -3.18108, nametomatch='Calton Hill', matchfilter=40)
    for result in images:
        assert isinstance(result, dict) and isinstance(result['image'], str)

def test_get_page_match():
    wiki = WikiExtractor(language='en', userinfo="test")
    suggested = wiki.get_page_match("Staines Moor", 51.43260, -0.51074, bestmatch=False, maxdistance=100)
    for page in suggested:
        assert isinstance(page['title'], str)

def test_get_page_text():
    wiki = WikiExtractor(language='en', userinfo="test")
    page = wiki.get_page_text('Staines Bridge', limit=500, translateto='de')
    assert isinstance(page['text'][0], str) and len(page['text']) == 1

def test_get_nearby_pages():
    wiki = WikiExtractor(language='en', userinfo="test")
    data = wiki.get_nearby_pages(54.6687, -7.6823)
    for page in data:
        assert isinstance(page['title'], str) and isinstance(page['coordinates']['lat'], float)



from wikigeo.wikisearch import WikiExtractor
from wikigeo.wikisource.wikitext import WikiText
import concurrent.futures
import time
import logging

class ConcurrentSearcher(object):

    """
    
    Runs search methods with multiple sets of parameters concurrently.  
    Keep max searches low so as not to request more than 50 pages in a minute (conservative estimate)
    
    """

    def __init__(self, language, userinfo, maxlimit=True):
        """

        for language code options: https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
        for info on user details see: https://www.mediawiki.org/wiki/API:Etiquette#The_User-Agent_header

        """
        self.wiki = WikiExtractor(language, userinfo)
        self.maxlimit = maxlimit
        self.language = language
    
    def multi_nearby_pages(self, coordpairs, limit=4, radiusmetres=10000):
        """
        
        gets nearby pages from multiple sets of coordinates.

        coordpairs: a list of tuples containing lat lon pairs

        limit: int, max number of pages to return

        returns a list of results. Each result is a dictionary with a 'result' key containing a list of dictionaries (one page per dictionary), with 'title', 'label', 'description', 'coordinates', 'image'.
        Results are outputted in the same order of the given coords.

        [{'coords': (lat, lon), 'result': [{result1}, {result2} ...]}, ...]
        
        """

        if(self.maxlimit):
            if(len(coordpairs) > 50):
                raise Exception("""Making more than 50 parallel requests is not advised. 
            See https://www.mediawiki.org/wiki/API:Etiquette#Request_limit for details
            and https://phabricator.wikimedia.org/ for advice on api limits. 
            To bypass add maxlimit=False as a parameter for ConcurrentSearcher""")

        lats = [coordpair[0] for coordpair in coordpairs]
        lons = [coordpair[1] for coordpair in coordpairs]
        limit = [limit for coordpair in coordpairs]
        radiusmetres = [radiusmetres for coordpair in coordpairs]
        with concurrent.futures.ThreadPoolExecutor() as executor:
            results = executor.map(self.wiki.get_nearby_pages, lats, lons, limit, radiusmetres)
        output = [{'coords': coordpair, 'result': result} for coordpair, result in zip(coordpairs, results)]
        return output
        

    def multi_page_text(self, titles, textlen, translateto):
        """
        
        Gets text from multiple pages by title.

        titles: list of wiki page titles

        textlen: optional, int representing character limit for text returned.

        translateto: string language code of language to translate text into e.g. 'en'. 
        See https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes for options.

        returns a list of dictionaries of headers and text for each page

        [{'title': inputtedtitle, 'text': textresult}, ...]
        
        """

        if(self.maxlimit):
            if(len(titles) > 50):
                raise Exception("""Making more than 50 parallel requests is not advised. 
            See https://www.mediawiki.org/wiki/API:Etiquette#Request_limit for details
            and https://phabricator.wikimedia.org/ for advice on api limits. 
            To bypass add maxlimit=False as a parameter for ConcurrentSearcher""")
        
        if(not isinstance(textlen, int)):
            raise Exception('invalid textlen argument; must be one of False or an integer')
        scraper = WikiText(textlen, self.language)
        with concurrent.futures.ThreadPoolExecutor() as executor:
            results = executor.map(scraper.scrape_page_text, titles)
        if(translateto):
            data = [result for result in results]
            translated = scraper.translate_text([record['text'] for record in data], translateto)
            output = [{'title': record['title'], 'text': translation} for record, translation in zip(data, translated)]
        else:
            output = [result for result in results]
        return output

    def multi_nearby_images(self, coordpairs, namestomatch=False, radiusmetres=10000, matchfilter=False):
        """
        
        Gets images nearby for each coord pair.

        coordpairs: a list of tuples containing lat lon pairs

        namestomatch: a list of names to match with each lat lon pair

        radiusmetres: an int max 10000, distance from coords to search (default 10000)

        matchfilter: either False or an int representing min name match value for results with a name to match (between 0 and 100)
        
        returns list of dictionary of results
        [{'coords: latlon, 'result': [result1, result 2, ...]}, ...]

        Notes: inputs of coords and names must in the same order to match and must be same length
        (if names are missing use False as placeholder)
        
        """
        
        if(self.maxlimit):
            if(len(coordpairs) > 50):
                raise Exception("""Making more than 50 parallel requests is not advised. 
            See https://www.mediawiki.org/wiki/API:Etiquette#Request_limit for details
            and https://phabricator.wikimedia.org/ for advice on api limits. 
            To bypass add maxlimit=False as a parameter for ConcurrentSearcher""")

        lats = [coordpair[0] for coordpair in coordpairs]
        lons = [coordpair[1] for coordpair in coordpairs]
        if(not namestomatch):
            namestomatch = [False for coordpair in coordpairs]
        radiusmetres = [radiusmetres for coordpair in coordpairs]
        matchfilters = []
        for name in namestomatch:
            if(name):
                matchfilters.append(matchfilter)
            else:
                matchfilters.append(False)

        with concurrent.futures.ThreadPoolExecutor() as executor:
            results = executor.map(self.wiki.get_nearby_images, lats, lons, radiusmetres, namestomatch, matchfilters)
        output = [{'coords': coordpair, 'result': result} for coordpair, result in zip(coordpairs, results)]
        return output

    def multi_page_match(self, searches, bestmatch=False, maxdistance=30, minnamematch=0):
        """
        
        Gets suggested page match for each keyword and coordpair in searchparams.

        searches: a list of tuples containing keyword (string), lat (float), lon (float)

        bestmatch: 

        bestmatch='name' will return the best (highest on scale 0-100) match based 
        on keyword matching page title 

        bestmatch='distance' will return the best (closest) match on distance

        bestmatch=False will return all geolocated matches nearby

        maxdistance: int, results must be less than the max distance in km of results from search coords (default 30)
        
        minnamematch: int, results must have a higher name match than minnamematch, between 0-100 (default 50)
        
        returns a list of dictionary/lists of dictionaries containing page title, 
        description, label, image, distance, coords and match rating
        
        [{'keyword': input, 'result': [{result1}, {result2}, ...]}, ...]

        """

        if(self.maxlimit):
            if(len(searches) > 50):
                raise Exception("""Making more than 50 parallel requests is not advised. 
            See https://www.mediawiki.org/wiki/API:Etiquette#Request_limit for details
            and https://phabricator.wikimedia.org/ for advice on api limits. 
            To bypass add maxlimit=False as a parameter for ConcurrentSearcher""")
        
        keywords = [search[0] for search in searches]
        lats = [search[1] for search in searches]
        lons = [search[2] for search in searches]
        bestmatch = [bestmatch for search in searches]
        maxdistance = [maxdistance for search in searches]
        minnamematch = [minnamematch for search in searches]
        logging.debug(keywords)
        with concurrent.futures.ThreadPoolExecutor() as executor:
            results = executor.map(self.wiki.get_page_match, keywords, lats, lons, bestmatch, maxdistance, minnamematch)
        output = [{'keyword': keyword, 'result': result} for keyword, result in zip(keywords, results)]
        return output



from wikigeo.wikisource.wikiapi import WikipediaAPI, WikiCommonsAPI

class TestWikiapi:

    def test_search_nearby(self):
        wiki = WikipediaAPI("testing marymcguire1718@gmail.com")
        wiki.search_nearby(51.43295, -0.5114918947219849, 10, 10000)
        result = wiki.return_data()
        print(result)
        assert isinstance(result['query']['pages'], dict)

    def test_search_string(self):
        wiki = WikipediaAPI("testing marymcguire1718@gmail.com")
        wiki.search_string('Staines Bridge', limit=5)
        result = wiki.return_data()
        print(result)
        assert isinstance(result['query']['pages'], dict)

    def test_commons_search_nearby_result(self):
        commons = WikiCommonsAPI("testing marymcguire1718@gmail.com")
        commons.search_nearby(51.43295, -0.5114918947219849, 1000)
        result = commons.return_data()
        print(result)
        assert isinstance(result['query']['pages'], dict)

    def test_commons_search_nearby_noresult(self):
        commons = WikiCommonsAPI("testing marymcguire1718@gmail.com")
        commons.search_nearby(45.78024, -34.91654, 1000)
        result = commons.return_data()
        print(result)
        assert isinstance(result, dict)




from requests_html import HTML, HTMLSession
from wikigeo.wikisearch import WikiExtractor
import time
import wikipedia

# test speed for scraping instead of using parse api


wiki = WikiExtractor(language='en', userinfo="DaytripApp (marymcguire1718@gmail.com)")


def scrape():
    domain = 'https://en.wikipedia.org/wiki/'
    title = 'Central London'
    url = domain + title.replace(' ', '_')
    session = HTMLSession()
    response = session.get(url)
    print(response)
    page = response.html
    pagetext = [para.text for para in page.find('p, h2, h3')]
    pagetext = ' '.join(pagetext).replace('[edit]', '')
    result = {'text': pagetext}
    return result

def run_get_page_text():
    paras = wiki.get_page_text('Central London')
    return paras

def wiki_mod():
    ny = wikipedia.page('Central London')
    return ny.content

results_run = []
results_scrape = []
results_mod = []

for i in range(7):
    start = time.time()
    run_get_page_text()
    end = time.time()
    results_run.append(end-start)

    start = time.time()
    scrape()
    end = time.time()
    results_scrape.append(end-start)

    start = time.time()
    wiki_mod()
    end = time.time()
    results_mod.append(end-start)

print('average for run:', sum(results_run)/len(results_run))
print('average for scrape:', sum(results_scrape)/len(results_scrape))
print('average for mod:', sum(results_mod)/len(results_mod))

"""
current winner: scrape

average for run: 0.5450178214481899
average for scrape: 0.25179593903677805
average for mod: 0.6314457484654018
"""








import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='wikigeo',
    version='0.0.3',
    description='Getting point of interest data by geographic coordinates from Wikipedia',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    author='Mary McGuire',
    author_email='marymcguire1718@gmail.com',
    install_requires=["requests_html",
    "googletrans", "fuzzywuzzy", "python-Levenshtein-wheels"],
    extras_require={"pytest": "pytest==6.0.1", "tox": "tox==3.19.0"},
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: Wiki"
    ],
    url='https://github.com/mmc1718/wikigeo'
)


import json
import time
from wikigeo.wikisource.wikiapi import WikipediaAPI, WikiCommonsAPI
from wikigeo.wikisource.wikitext import WikiText
import logging
from fuzzywuzzy import fuzz
import math


class WikiExtractor(object):

    """
    
    Search wikipedia and return specified information.
    
    """

    def __init__(self, language, userinfo):
        self.user = userinfo
        self.language = language

    def get_nearby_pages(self, lat, lon, limit=4, radiusmetres=10000):
        """
        
        Get details for all pages within a given radius of given coordinates.

        radiusmeters: the distance in metres to search, can be max 10000 (10km). default is 10000
        
        limit: max number of pages to return (default 4)

        returns list of dictionaries with title, label, description, coordinates, image
        
        """

        titles = []
        labels = []
        descriptions = []
        coordinates = []
        images = []
        wiki = WikipediaAPI(self.user, self.language)
        wiki.search_nearby(lat, lon, limit, radiusmetres)
        response = wiki.return_data()
        for page, result in response['query']['pages'].items():
            titles.append(result['title'])
            try:
                labels.append(result['terms']['label'])
            except:
                labels.append(None)
            try:
                descriptions.append(result['terms']['description'])
            except:
                descriptions.append(None)
            coordinates.append({'lat': result['coordinates'][0]['lat'], 'lon': result['coordinates'][0]['lon']})
            try:
                thumbnail = result['thumbnail']['source']
                image = thumbnail.split('/')
                image.pop(-1)
                image.remove('thumb')
                imageurl = '/'.join(image)
                images.append(imageurl)
            except:
                images.append(None)
            output = [
                {'title': title, 'description': description, 'coordinates': latlon, 'label': label, 'image': image}
                 for title, description, latlon, label, image
                  in zip(titles, descriptions, coordinates, labels, images)]
            return output
        

    def get_page_text(self, pagetitle, limit=False, translateto=False):
        """
        
        Retrieve the full text of a given page by page title.
        
        pagetitle: exact title of page to scrape
        
        limit: int, character limit of text returned. If set to False then the full text is returned. (default is False).
        
        translateto: language code for language that text should be translated into.
        
        returns a dictionary with pagetitle and text.

        Note: if requesting translated text, limit should be a max of 10000

        """
            
        wiki = WikiText(limit, self.language)
        result = wiki.scrape_page_text(pagetitle)
        if(translateto):
            result['text'] = wiki.translate_text([result['text']], translateto)
        return result

    def get_nearby_images(self, lat, lon, radiusmetres=10000, nametomatch=False, matchfilter=False):
        """
        
        Gets all images nearby given coordinates from Wikimedia Commons.

        radiusmetres: distance to search, max 10000 (10km)
        
        nametomatch: if set to a string then results are given a match rating against the string. If False then no matching is done.
        
        matchfilter: if set to an int then results are filtered for results that have a higher rated name match,
        can be between 0 (no matching) and 100 (exact match only)
        
        returns list of dictionaries containing image data
        [{'image': i, 'title': t, 'url': url, 'name match': x}, ...]
        
        Note: if using matchfilter, nametomatch must be set to a string
        
        """

        commons = WikiCommonsAPI(self.user)
        if(matchfilter and (not nametomatch)):
            raise Exception('nametomatch must be set to a name if using a matchfilter')
        commons.search_nearby(lat, lon, radiusmetres)
        response = commons.return_data()
        try:
            images = response['query']['pages']
            imagedata = []
            for key, value in images.items():
                image = value['imageinfo'][0]['url']
                title = value['title']
                url = value['imageinfo'][0]['descriptionurl']
                imagedata.append({'image': image, 'title': title, 'url': url})
        except KeyError:
            imagedata.append(None)
        if(nametomatch):
            for image in imagedata:
                image['name match'] = fuzz.partial_ratio(nametomatch, image['title'])
            imagedata.sort(key=lambda x: int(x['name match']), reverse=True)
        if(matchfilter):
            imagedata = [image for image in imagedata if image['name match'] > matchfilter]
        return imagedata


    def _get_km_distance(self, lat1, lon1, lat2, lon2):
        """converts decimal distance into kms"""

        latA = float(lat1) * math.pi / 180
        lngA = float(lon1) * math.pi / 180
        latB = float(lat2) * math.pi / 180
        lngB = float(lon2) * math.pi / 180
        distance = 6378 * math.acos(math.cos(latA) * math.cos(latB) * math.cos(lngB - lngA) + math.sin(latA) * math.sin(latB))
        return distance

    def get_page_match(self, keyword, searchlat, searchlon, bestmatch, maxdistance=30, minnamematch=0):
        """
        
        Searches for all geolocated wiki pages containing given keyword.
        Filters to results that are less than the maxdistance (default is 30km)

        bestmatch: 

        bestmatch='name' will return the best (highest on scale 0-100) match based 
        on keyword matching page title 

        bestmatch='distance' will return the best (closest) match on distance

        bestmatch=False will return all geolocated matches nearby
        
        maxdistance: int, results must be less than the max distance in km of results from search coords 
                    (default 30)
        
        minnamematch: int, results must have a higher name match than minnamematch, between 0-100 
                    (default 50)

        returns a (list of) dictionaries containing page title, description, label, image, distance, coords
        and match rating

        Note: Works best for unique, proper names of geographically located places (e.g. landmarks, buildings, parks)
        Bases location on centrepoint, so points on the edge of very large/spread out entities may not match.
        
        """

        data = []
        wiki = WikipediaAPI(self.user, self.language)
        wiki.search_string(keyword, limit=5)
        search_results = wiki.return_data()
        try:
            logging.debug('results: ' + str(len(search_results['query']['pages'])))
        except KeyError:
            logging.warning('cannot find data from ' + str(search_results))
        for page, info in search_results['query']['pages'].items():
            name = info['title']
            try:
                lat = info['coordinates'][0]['lat']
                lon = info['coordinates'][0]['lon']
            except KeyError:
                logging.debug('cannot find coords for ' + name)
                continue
            try:
                description = info['terms']['description']
            except KeyError:
                logging.debug('cannot find description for ' + name)
                description = None
            try:
                label = info['terms']['label']
            except KeyError:
                logging.debug('cannot find label for ' + name)
                label = None
            try:
                image = info['original']['source']
            except KeyError:
                logging.debug('cannot find image for ' + name)
                image = None
            result = {'title': name, 'description': description, 'label': label, 'image': image, 'lat': lat, 'lon': lon}
            result['distance'] = self._get_km_distance(searchlat, searchlon, lat, lon)
            result['name match'] = fuzz.ratio(name, keyword)
            #result['summary'] = self.getPageText([name])['text'][0:200]
            data.append(result)
        logging.debug('final data length: ' + str(len(data)))
        logging.debug('results with coords saved from wiki search: ' + str(len(data)))
        # filtering for relevant results
        #results = [place for place in data if ((place['distance'] < 10 and place['name match'] > 50) or (place['name match'] > 65 and place['distance'] < 30) or (place['distance'] < 5))]
        results = [place for place in data if ((place['distance'] < maxdistance) and (place['name match'] > minnamematch))]
        if(results):
            # getting best match if requested
            if(bestmatch == 'name'):
                results.sort(key=lambda result: int(result['name match']), reverse=True)
                logging.debug('wikis: ' + str(results))
                return results[0]
            elif(bestmatch == 'distance'):
                results.sort(key=(lambda result: abs(result['distance'])))
                logging.debug('wikis: ' + str(results))
                return results[0]
            else:
                return results
        else:
            return []

